package org.processmining.glueminer.dialogs;

import javax.swing.JComponent;

import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;
import org.processmining.framework.util.ui.wizard.ProMWizardStep;
import org.processmining.glueminer.parameters.GlueMinerConstants;
import org.processmining.glueminer.parameters.GlueMinerParameters;

import com.fluxicon.slickerbox.components.NiceDoubleSlider;
import com.fluxicon.slickerbox.components.NiceIntegerSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class GlueMinerWizardStep extends ProMPropertiesPanel implements ProMWizardStep<GlueMinerParameters> {
	public static String TITLE = "Configure Glue Miner parameters";
	
	NiceDoubleSlider placeFitnessLevel;
	NiceDoubleSlider maxUnderfednessLevel;
	NiceDoubleSlider netFitnessLevel;
	NiceDoubleSlider relThresholdOnStartAndEndActivities;
	NiceDoubleSlider relThresholdOnPlacesLogSize;
	NiceIntegerSlider maxSourceOrTargetTransitions;
	NiceIntegerSlider minAcceptedBehaviorCount;
	
	public GlueMinerWizardStep(String title) {
		super(title);
		// TODO Auto-generated constructor stub
	}
	
	public GlueMinerWizardStep(XLog log) {
		super(TITLE);
		
		this.placeFitnessLevel = SlickerFactory.instance().createNiceDoubleSlider("Place fitness level", GlueMinerConstants.minPlaceFitnessLevel, GlueMinerConstants.maxPlaceFitnessLevel, GlueMinerConstants.defaultPlaceFitnessLevel, Orientation.HORIZONTAL);
		this.maxUnderfednessLevel = SlickerFactory.instance().createNiceDoubleSlider("Max underfedness level", GlueMinerConstants.minMaxUnderfednessLevel, GlueMinerConstants.maxMaxUnderfednessLevel, GlueMinerConstants.defaultMaxUnderfednessLevel, Orientation.HORIZONTAL);
		this.netFitnessLevel = SlickerFactory.instance().createNiceDoubleSlider("Petrinet fitness level", GlueMinerConstants.minMinNetFitness, GlueMinerConstants.maxMinNetFitness, GlueMinerConstants.defaultMinNetFitness, Orientation.HORIZONTAL);
		this.relThresholdOnStartAndEndActivities = SlickerFactory.instance().createNiceDoubleSlider("Threshold on start/end activities", GlueMinerConstants.minRelativeThresholdOnStartAndEndActivities, GlueMinerConstants.maxRelativeThresholdOnStartAndEndActivities, GlueMinerConstants.defaultRelativeThresholdOnStartAndEndActivities, Orientation.HORIZONTAL);
		this.relThresholdOnPlacesLogSize = SlickerFactory.instance().createNiceDoubleSlider("Threshold on places support", GlueMinerConstants.minRelativeThresholdOnPlacesRegardingLogSize, GlueMinerConstants.maxRelativeThresholdOnPlacesRegardingLogSize, GlueMinerConstants.defaultRelativeThresholdOnPlacesRegardingLogSize, Orientation.HORIZONTAL);
		this.maxSourceOrTargetTransitions = SlickerFactory.instance().createNiceIntegerSlider("Max source or target transitions per place", GlueMinerConstants.minMaxSourceOrTargetTransitions, GlueMinerConstants.maxMaxSourceOrTargetTransitions, GlueMinerConstants.defaultMaxSourceOrTargetTransitions, Orientation.HORIZONTAL);
		this.minAcceptedBehaviorCount = SlickerFactory.instance().createNiceIntegerSlider("Keep only behavior with at least this count", GlueMinerConstants.minMinAcceptedBehaviorCount, GlueMinerConstants.maxMinAcceptedBehaviorCount, GlueMinerConstants.defaultMinAcceptedBehaviorCount, Orientation.HORIZONTAL);
		
		this.add(this.placeFitnessLevel);
		this.add(this.maxUnderfednessLevel);
		this.add(this.netFitnessLevel);
		this.add(this.relThresholdOnStartAndEndActivities);
		this.add(this.relThresholdOnPlacesLogSize);
		this.add(this.maxSourceOrTargetTransitions);
		this.add(this.minAcceptedBehaviorCount);
	}

	public GlueMinerParameters apply(GlueMinerParameters model, JComponent component) {
		// TODO Auto-generated method stub
		GlueMinerParameters parameters = new GlueMinerParameters(this.placeFitnessLevel.getValue(), this.maxUnderfednessLevel.getValue(), this.netFitnessLevel.getValue(), this.relThresholdOnStartAndEndActivities.getValue(), this.relThresholdOnPlacesLogSize.getValue(), this.maxSourceOrTargetTransitions.getValue(), this.minAcceptedBehaviorCount.getValue());
		
		return parameters;
	}

	public boolean canApply(GlueMinerParameters model, JComponent component) {
		// TODO Auto-generated method stub
		return true;
	}

	public JComponent getComponent(GlueMinerParameters model) {
		// TODO Auto-generated method stub
		return this;
	}

	public String getTitle() {
		// TODO Auto-generated method stub
		return TITLE;
	}

}
