package org.processmining.glueminer.models;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class DirectlyFollowsMatrix {
	Map<String, Map<String, Integer>> directlyFollowsMatrix;
	Map<String, Integer> maxSuccessOcc;
	Map<String, Map<String, Integer>> invDirectlyFollowsMatrix;
	Map<String, Integer> maxPrecedOcc;
	
	public DirectlyFollowsMatrix() {
		directlyFollowsMatrix = new HashMap<String, Map<String, Integer>>();
		invDirectlyFollowsMatrix = new HashMap<String, Map<String, Integer>>();
		maxSuccessOcc = new HashMap<String, Integer>();
		maxPrecedOcc = new HashMap<String, Integer>();
	}
	
	public void addRelationshipBetweenActivities(String activity1, String activity2) {
		if (!directlyFollowsMatrix.containsKey(activity1)) {
			directlyFollowsMatrix.put(activity1, new HashMap<String, Integer>());
		}
		if (!maxSuccessOcc.containsKey(activity1)) {
			maxSuccessOcc.put(activity1, 0);
		}
		if (!invDirectlyFollowsMatrix.containsKey(activity2)) {
			invDirectlyFollowsMatrix.put(activity2, new HashMap<String, Integer>());
		}
		if (!maxPrecedOcc.containsKey(activity2)) {
			maxPrecedOcc.put(activity2, 0);
		}
		if (!directlyFollowsMatrix.get(activity1).containsKey(activity2)) {
			directlyFollowsMatrix.get(activity1).put(activity2, 0);
		}
		if (!invDirectlyFollowsMatrix.get(activity2).containsKey(activity1)) {
			invDirectlyFollowsMatrix.get(activity2).put(activity1, 0);
		}
		directlyFollowsMatrix.get(activity1).put(activity2, directlyFollowsMatrix.get(activity1).get(activity2) + 1);
		maxSuccessOcc.put(activity1, Math.max(maxSuccessOcc.get(activity1), directlyFollowsMatrix.get(activity1).get(activity2)));
		invDirectlyFollowsMatrix.get(activity2).put(activity1, invDirectlyFollowsMatrix.get(activity2).get(activity1) + 1);
		maxPrecedOcc.put(activity2, Math.max(maxPrecedOcc.get(activity2), invDirectlyFollowsMatrix.get(activity2).get(activity1)));
	}
	
	public void removeActivitiesBelowSupport(Integer minSupportForPlaces) {
		List<String> directlyFollowsMatrixKeys = new ArrayList<String>(directlyFollowsMatrix.keySet());
		for (String key1 : directlyFollowsMatrixKeys) {
			List<String> subkeys = new ArrayList<String>(directlyFollowsMatrix.get(key1).keySet());
			for (String key2 : subkeys) {
				if (directlyFollowsMatrix.get(key1).get(key2) < minSupportForPlaces) {
					directlyFollowsMatrix.get(key1).remove(key2);
				}
			}
		}
		List<String> invDirectlyFollowsMatrixKeys = new ArrayList<String>(invDirectlyFollowsMatrix.keySet());
		for (String key1 : invDirectlyFollowsMatrixKeys) {
			List<String> subkeys = new ArrayList<String>(invDirectlyFollowsMatrix.get(key1).keySet());
			for (String key2 : subkeys) {
				if (invDirectlyFollowsMatrix.get(key1).get(key2) < minSupportForPlaces) {
					invDirectlyFollowsMatrix.get(key1).remove(key2);
				}
			}
		}
	}
	
	public void clean(Double threshold) {
		List<String> directlyFollowsMatrixKeys = new ArrayList<String>(directlyFollowsMatrix.keySet());
		for (String key1 : directlyFollowsMatrixKeys) {
			Integer maxOcc = maxSuccessOcc.get(key1);
			List<String> subkeys = new ArrayList<String>(directlyFollowsMatrix.get(key1).keySet());
			for (String key2 : subkeys) {
				if (directlyFollowsMatrix.get(key1).get(key2) < (maxOcc * (1.0 - threshold))) {
					directlyFollowsMatrix.get(key1).remove(key2);
				}
			}
		}
		List<String> invDirectlyFollowsMatrixKeys = new ArrayList<String>(invDirectlyFollowsMatrix.keySet());
		for (String key1 : invDirectlyFollowsMatrixKeys) {
			Integer maxOcc = maxPrecedOcc.get(key1);
			List<String> subkeys = new ArrayList<String>(invDirectlyFollowsMatrix.get(key1).keySet());
			for (String key2 : subkeys) {
				if (invDirectlyFollowsMatrix.get(key1).get(key2) < (maxOcc * (1.0 - threshold))) {
					invDirectlyFollowsMatrix.get(key1).remove(key2);
				}
			}
		}
	}
	
	public Integer addEvidentRelations(PlacesMemorizer memorizer, Set<String> possibleLeftActivitiesForPlacesAlgorithm, Set<String> possibleRightActivitiesForPlacesAlgorithm, Integer minSupportForPlaces, Map<String, Integer> activitiesCount) {
		Integer evidentRelationsCount = 0;
		Set<String> examinedActivitiesOnTheLeft = new HashSet<String>();
		Set<String> examinedActivitiesOnTheRight = new HashSet<String>();
		
		// simple directly follows relationships
		for (String key1 : directlyFollowsMatrix.keySet()) {
			if (directlyFollowsMatrix.get(key1).size() == 1) {
				String key2 = new ArrayList<String>(directlyFollowsMatrix.get(key1).keySet()).get(0);
				if (!key2.equals(key1)) {
					if (invDirectlyFollowsMatrix.get(key2).size() == 1) {
						if (possibleLeftActivitiesForPlacesAlgorithm.contains(key1) && possibleRightActivitiesForPlacesAlgorithm.contains(key2)) {
							Integer activityCount1 = activitiesCount.get(key1);
							Integer activityCount2 = activitiesCount.get(key2);
							boolean finalCondition = (activityCount1 > minSupportForPlaces && activityCount2 > minSupportForPlaces);
							
							if (finalCondition) {
								evidentRelationsCount++;
								memorizer.addPlace(key1, key2);
								examinedActivitiesOnTheLeft.add(key1);
								examinedActivitiesOnTheRight.add(key2);
							}
						}
					}
				}
			}
		}
		return evidentRelationsCount;
	}
	
	public void printResume() {
		System.out.println("directlyFollowsMatrix=");
		System.out.println(directlyFollowsMatrix);
		System.out.println("maxSuccessOcc=");
		System.out.println(maxSuccessOcc);
		System.out.println("invDirectlyFollowsMatrix=");
		System.out.println(invDirectlyFollowsMatrix);
		System.out.println("maxPrecedOcc=");
		System.out.println(maxPrecedOcc);
	}
}
