package org.processmining.glueminer.models;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class PlacesMemorizer {
	static String intraSetDelimiter = "@@@";
	static String interSetDelimiter = "###";
	
	Set<String> allInputActivitiesInPlaces;
	Set<String> allOutputActivitiesInPlaces;
	Map<String, Set<String>> inputActivitiesInPlaces;
	Map<String, Set<String>> outputActivitiesInPlaces;
	Map<String, Set<String>> parallelPlaces;
	Map<String, Set<String>> inputActivitiesMergeParallelPlaces;
	Map<String, Set<String>> outputActivitiesMergeParallelPlaces;
	
	public PlacesMemorizer() {
		allInputActivitiesInPlaces = new HashSet<String>();
		allOutputActivitiesInPlaces = new HashSet<String>();
		inputActivitiesInPlaces = new HashMap<String, Set<String>>();
		outputActivitiesInPlaces = new HashMap<String, Set<String>>();
		parallelPlaces = new HashMap<String, Set<String>>();
		inputActivitiesMergeParallelPlaces = new HashMap<String, Set<String>>();
		outputActivitiesMergeParallelPlaces = new HashMap<String, Set<String>>();
	}
	
	public Set<String> getAllInputActivitiesInPlaces() {
		return this.allInputActivitiesInPlaces;
	}
	
	public Set<String> getAllOutputActivitiesInPlaces() {
		return this.allOutputActivitiesInPlaces;
	}
	
	public void addPlace(String sourceActivity, String targetActivity) {
		Set<String> sourceActivities = new HashSet<String>();
		sourceActivities.add(sourceActivity);
		Set<String> targetActivities = new HashSet<String>();
		targetActivities.add(targetActivity);
		this.addPlace(sourceActivities, targetActivities);
	}
	
	public void addPlace(String sourceActivity, Set<String> targetActivities) {
		Set<String> sourceActivities = new HashSet<String>();
		sourceActivities.add(sourceActivity);
		this.addPlace(sourceActivities, targetActivities);
	}
	
	public void addPlace(Set<String> sourceActivities, String targetActivity) {
		Set<String> targetActivities = new HashSet<String>();
		targetActivities.add(targetActivity);
		this.addPlace(sourceActivities, targetActivities);
	}
	
	public String getPlaceLabel(Set<String> sourceActivities, Set<String> targetActivities) {
		List<String> sourceActivitiesAsList = new ArrayList<String>(sourceActivities);
		Collections.sort(sourceActivitiesAsList);
		String sourceActivitiesAsString = StringUtils.join(sourceActivitiesAsList, intraSetDelimiter);
		List<String> targetActivitiesAsList = new ArrayList<String>(targetActivities);
		Collections.sort(targetActivitiesAsList);
		String targetActivitiesAsString = StringUtils.join(targetActivitiesAsList, intraSetDelimiter);
		String placeString = sourceActivitiesAsString + interSetDelimiter + targetActivitiesAsString;
		return placeString;
	}
	
	public void addPlace(Set<String> sourceActivities, Set<String> targetActivities) {
		allInputActivitiesInPlaces.addAll(sourceActivities);
		allOutputActivitiesInPlaces.addAll(targetActivities);
		
		if (!checkRedundancy(sourceActivities, targetActivities)) {
			//System.out.println("adding place sourceActivities="+sourceActivities+" targetActivities="+targetActivities);
			String placeString = getPlaceLabel(sourceActivities, targetActivities);
			inputActivitiesInPlaces.put(placeString, new HashSet<String>(sourceActivities));
			outputActivitiesInPlaces.put(placeString, new HashSet<String>(targetActivities));
			parallelPlaces.put(placeString, new HashSet<String>());
			inputActivitiesMergeParallelPlaces.put(placeString, new HashSet<String>(sourceActivities));
			outputActivitiesMergeParallelPlaces.put(placeString, new HashSet<String>(targetActivities));
			
			for (String place : parallelPlaces.keySet()) {
				if (!place.equals(placeString)) {
					Set<String> inputActivitiesOtherPlace = new HashSet<String>(inputActivitiesInPlaces.get(place));
					Set<String> outputActivitiesOtherPlace = new HashSet<String>(outputActivitiesInPlaces.get(place));
					
					inputActivitiesOtherPlace.retainAll(sourceActivities);
					outputActivitiesOtherPlace.retainAll(targetActivities);
					
					if (inputActivitiesOtherPlace.size() == 0 && outputActivitiesOtherPlace.size() == 0) {
						parallelPlaces.get(placeString).add(place);
						parallelPlaces.get(place).add(placeString);
						inputActivitiesMergeParallelPlaces.get(placeString).addAll(inputActivitiesMergeParallelPlaces.get(place));
						inputActivitiesMergeParallelPlaces.get(place).addAll(inputActivitiesMergeParallelPlaces.get(placeString));
						outputActivitiesMergeParallelPlaces.get(placeString).addAll(outputActivitiesMergeParallelPlaces.get(place));
						outputActivitiesMergeParallelPlaces.get(place).addAll(outputActivitiesMergeParallelPlaces.get(placeString));
					}
				}
			}
		}
		else {
			//System.out.println("proposed place sourceActivities="+sourceActivities+" targetActivities="+targetActivities+" is redundant");
		}
	}
	
	public boolean checkRedundancy(Set<String> sourceActivities, Set<String> targetActivities) {
		for (String place : inputActivitiesMergeParallelPlaces.keySet()) {
			if (inputActivitiesMergeParallelPlaces.get(place).containsAll(sourceActivities)) {
				if (outputActivitiesMergeParallelPlaces.containsKey(place) && outputActivitiesMergeParallelPlaces.get(place).containsAll(targetActivities)) {
					return true;
				}
			}
		}
		return false;
	}
	
	public void addToPetrinet(Petrinet net, Map<String, Transition> activitiesTransitions) {
		for (String place : this.inputActivitiesInPlaces.keySet()) {
			List<String> sourceActivities = new ArrayList<String>(this.inputActivitiesInPlaces.get(place));
			List<String> targetActivities = new ArrayList<String>(this.outputActivitiesInPlaces.get(place));
			
			Place p = net.addPlace(place);
			
			for (String sourceActivity : sourceActivities) {
				Transition sourceTransition = activitiesTransitions.get(sourceActivity);
				net.addArc(sourceTransition, p);
			}
			
			for (String targetActivity : targetActivities) {
				Transition targetTransition = activitiesTransitions.get(targetActivity);
				net.addArc(p, targetTransition);
			}
		}
	}
	
	public void printResume() {
		System.out.println("allInputActivitiesInPlaces=");
		System.out.println(allInputActivitiesInPlaces);
		System.out.println("allOutputActivitiesInPlaces=");
		System.out.println(allOutputActivitiesInPlaces);
		System.out.println("inputActivitiesInPlaces=");
		System.out.println(inputActivitiesInPlaces);
		System.out.println("outputActivitiesInPlaces=");
		System.out.println(outputActivitiesInPlaces);
		System.out.println("parallelPlaces=");
		System.out.println(parallelPlaces);
		System.out.println("inputActivitiesMergeParallelPlaces=");
		System.out.println(inputActivitiesMergeParallelPlaces);
		System.out.println("outputActivitiesMergeParallelPlaces=");
		System.out.println(outputActivitiesMergeParallelPlaces);
	}
	
	public static void main(String[] args) {
		PlacesMemorizer memorizer = new PlacesMemorizer();
		memorizer.addPlace("A", "B");
		memorizer.addPlace("C", "D");
		memorizer.addPlace("E", "D");
		
		Set<String> thisInputActivities = new HashSet<String>();
		Set<String> thisOutputActivities = new HashSet<String>();
		
		thisInputActivities.add("A");
		thisInputActivities.add("C");
		
		thisOutputActivities.add("B");
		thisOutputActivities.add("D");
		
		memorizer.addPlace(thisInputActivities, thisOutputActivities);
		//memorizer.printResume();
	}
}
