package org.processmining.glueminer.plugins;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginQuality;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.glueminer.models.GeneratedPetrinet;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;

@Plugin(name = "Extract Petri net from generated Petri net", level = PluginLevel.NightlyBuild, parameterLabels = { "Generated Petrinet" }, returnLabels = { "Petrinet", "Initial marking", "Final marking" }, returnTypes = { Petrinet.class, Marking.class, Marking.class }, help = "No help", quality = PluginQuality.VeryGood, categories = {
		PluginCategory.Discovery })
public class PetrinetFromGeneratedPetrinet {
	@UITopiaVariant(affiliation = "PADS", author = "Alessandro Berti", email = "a.berti@pads.rwth-aachen.de")
	@PluginVariant(requiredParameterLabels = { 0 })
	public static Object[] extractPetrinet(PluginContext context, GeneratedPetrinet gnet) {
		Petrinet net = gnet.net;
		Marking initialMarking = gnet.initialMarking;
		Marking finalMarking = gnet.finalMarking;
		
		context.getConnectionManager().addConnection(new InitialMarkingConnection(net, initialMarking));
		context.getConnectionManager().addConnection(new FinalMarkingConnection(net, finalMarking));
		
		Object[] objects = { net, initialMarking, finalMarking };
		
		return objects;
	}
}
